/* a temporary tool until we can execute sksh commands from within make */

/*\
 *
 * SYNOPSIS
 *
 *     subdir_do [d dir] mode command1 [++ command2 [++ command3]] -- subdir1 subdir2 subdir3 ...
 *
 * ARGUMENTS
 *
 *	d dir	If present, subdir_do will cd to dir before doing anything else.
 *
 *	mode	mode of operation
 *		s == subdir_do, change cd to each subdir before running commands
 *		0 == just execute the commands, with subdir expansion in the commands
 *
 *	commandn
 *		a command to execute
 *		The command to execute can also contain one or more % characters,
 *		which will be substituted with the name of the current subdir
 *		directory (NB: not the current working directory!)
 *
 *	++	to add another command to be executed after the previous one.
 *		up to 10 commands can be given.
 *
 *	--	delimiter between commands and subdir names
 *
 *	subdirn any number of subdirectories to cd to and perform the commands
 *		in if in `s' mode. Substitution of the % character to the active
 *		subdir is performed in both modes. If in `0' mode, no cd takes
 *		place.
 *
 * AUTHOR
 *
 *	Leonard Norrgard, Soft Service, Inc. E-Mail: vinsci@soft.fi
 *
 * HISTORY
 *	Sat 29-Dec-90 / ln / Wrote original version.
 *	Tue 01-Jan-91 / ln / Updated error_usage() to reflect d option.
 * 
\*/

/* compile with lattice 5.05! */

#include <stdio.h>
#include <stdlib.h>

/* the number of commands allowed to perform per iteration of subdir */
#define NCMDS 10

/* command line length (note that some makefiles generate very long lines) */
#define CMDLEN 1000

/* holds argv[] for error routines */
char **xargv;

void
error(char *fmt, void *arg1, void *arg2, void *arg3)
{
 fprintf(stderr, fmt, arg1, arg2, arg3);
 exit(1);
}

void
error_usage()
{
  fprintf(stderr, "Usage:\n%s [d dir] mode cmd1 [++ cmd2 [++ ...]] -- dir1 dir2 dir3 ...\n", xargv[0]);
  exit(1);
}

char *
expand_command(char *cmd, char *dir)
{
#define XLEN (2*CMDLEN)
  static char xcmdbuf[XLEN];  /* buffer to expand command in */
  char *wrk = xcmdbuf;

  /* clear the buffer first, so we needn't worry about terminating the strings */
  memset(wrk, '\0', XLEN);

  while((wrk < &xcmdbuf[XLEN-1]) && *cmd) {
	if (*cmd!='%')
		*wrk++ = *cmd++;
	else {
		strcat(wrk, dir);
		while (*wrk)
			++wrk;
		++cmd;
	}
  }
  if (wrk == &xcmdbuf[XLEN-1])
	error("%s: Expansion buffer too small for expanding command:\n%s\n", xargv[0], cmd, 0);

  return xcmdbuf;
}

int
main(int argc, char *argv[])
{
 static char cmd[NCMDS][CMDLEN];
 char *xcmd; /* expanded command */
 int cmdno=0, maxcmd;
 int curargv;
 int work=1;
 char mode='\0';
 #define CDBUFSIZ 300
 static char cdbuf[CDBUFSIZ];
 xargv = argv;

 if (argc < 5 || argv[1][0]=='?')
	error_usage();

 /* skip program name */
 curargv = 1;

 if(getcwd(cdbuf, CDBUFSIZ)==NULL)
	error("%s: Couldn't find current working directory.\n", argv[0], 0, 0);

 mode = argv[curargv][0];

 if (mode=='d') {
	if (0 != chdir(argv[++curargv]))
		error("%s: Couldn't change directory to `%s'\n", argv[0], argv[curargv], 0);
	++curargv;
	mode = argv[curargv][0];
 }

 if ((mode!='s') && (mode != '0'))
	error_usage();

 ++curargv;

 while(work && (curargv <= argc)) {
     /* remember boolean logic? ;-) */
     if ((argv[curargv][0]!='-') || (argv[curargv][1]!='-')) {
	     if ((argv[curargv][0]!='+') || (argv[curargv][1]!='+')) {
		strcat(cmd[cmdno], argv[curargv]);
		strcat(cmd[cmdno], " ");
		++curargv;
	     }
	     else {
		++curargv;
		++cmdno;
		if (cmdno == NCMDS)
			error("%s: Too many commands. Only %u are allowed.\nChange NCMDS and recompile!\n", argv[0], (void *) NCMDS, 0);
	     }
     }
     else {
	work = 0;
	++curargv;
     }
 }

 maxcmd = cmdno;

 while (curargv < argc) {
	if (mode=='s')
		if (0 != chdir(argv[curargv]))
			error("%s: Couldn't change directory to `%s'\n", argv[0], argv[curargv], 0);

	fprintf(stdout, "\t\t    subdir %s\n", argv[curargv]);

	for (cmdno=0; cmdno<=maxcmd; ++cmdno) {
		xcmd = expand_command(cmd[cmdno], argv[curargv]);

		if (0 != system(xcmd))
			error("%s: Couldn't execute command `%s'\nIn directory %", argv[0], cmd[cmdno], argv[curargv]);
	}

	if (mode=='s')
		if (0 != chdir("/"))
			error("%s: Couldn't change dir to parent from %s", argv[0], argv[curargv], 0);

	++curargv;
 }

 if (0 != chdir(cdbuf))
	error("%s: Couldn't change cd back to initial dir `%s'\n", argv[0], cdbuf, 0);

 return 0;
}




